const { _, log } = require('./basd')

// function mixin(target, ...sources) {
//   Object.assign(target.prototype, ...sources)
// }

class Mixin {
  static getSource(src) {
    return _.isFn(src) ? src.prototype : _.get(src, 'constructor.prototype', src)
  }
  static assign(target, src) {
    // log(Object.getOwnPropertyNames(src.constructor.prototype))
    for (const key of Object.getOwnPropertyNames(src)) {
      const info = Object.getOwnPropertyDescriptor(src, key)
      Object.defineProperty(target, key, info)
    }
  }
  static getFields(src, ignore) {
    ignore = _.toArr(ignore)
    src = this.getSource(src)
    return Object.getOwnPropertyNames(src).filter(f => !['constructor'].concat(ignore).includes(f))
  }
  static extend(target, src, ignore) {
    src = this.getSource(src)
    const fields = this.getFields(src, ignore)
    for (const key of fields) {
      const info = Object.getOwnPropertyDescriptor(src, key)
      if (info.value)
        Object.defineProperty(target, key, { value: src[key].bind(target), configurable: true, writable: true })
      else {
        const field = { configurable: true }
        if (info.get)
          field.get = info.get.bind(target)
        if (info.set)
          field.set = info.set.bind(target)
        if (!_.isEmpty(field))
          Object.defineProperty(target, key, field)
      }
    }
  }
}

module.exports = Mixin
