const { _, log } = require('./basd')

class Mu {
  constructor(ctx) {
    _.objProp(this, 'ctx', ctx)
    this.topics = {}
    this.sets = {}
  }
  _parse(topic) {
    let parts = topic.split('_')
    if (parts.length === 1)
      return { topic, method: topic }
    if (parts.length > 2)
      throw new Error('topic can only have 2 levels')
    topic = parts[0]
    let method = parts[1]
    parts = topic.split('2')
    if (parts.length === 1)
      return { topic, method }
    if (parts.length > 2)
      throw new Error('binary is only 2')
    if (method === parts[0])
      return { topic, method, binary: true, alpha: true }
    return { topic, method, binary: true }
  }
  use(topic, tag, fn) {
    let obj = this._parse(topic)
    let arr = _.get(this.topics, [obj.topic, obj.method, tag], [])
    arr.push(fn)
    _.set(this.topics, [obj.topic, obj.method, tag], arr)
    let set = _.keys(_.get(this.topics, [obj.topic, obj.method]))
    _.set(this.sets, [obj.topic, obj.method], set)
  }
  freeze() {
    for (let topic in this.sets) {
      for (let method in this.sets[topic]) {
        let tmp = topic === method ? topic : [topic, method].join('_')
        let obj = this._parse(tmp)
        if (!obj.binary || obj.alpha)
          continue
        let parts = obj.topic.split('2')
        let order = this.order(obj.topic, parts[0])
        order = _.reverse(order)
        this.reorder(obj.topic, obj.method, order)
      }
    }
  }
  apply(topic, data, ctx, tags) {
    const run = (fn, data, ctx) => {
      if (ctx)
        fn = fn.bind(ctx)
      return fn(data, ctx)
    }
    ctx = ctx || this.ctx
    const obj = this._parse(topic)
    let order
    if (tags)
      order = this.order(obj.topic, obj.method)
    else
      order = this.order(obj.topic, obj.method)
    const entry = _.get(this.topics, [obj.topic, obj.method])
    const firstTag = order[0]
    const firstFns = _.get(this.topics, [obj.topic, obj.method, firstTag], [data => data]).slice()
    const firstFn = firstFns.shift()
    const first = run(firstFn, data, ctx)
    let result = first
    if (first instanceof Promise) {
      const goAsync = async () => {
        result = await result
        try {
          for (const tag of order)
            for (const fn of _.get(this.topics, [obj.topic, obj.method, tag], []))
              result = await run(fn, result, ctx)
          return result
        } catch (err) { throw err }
      }
      return goAsync(first, ctx)
    }
    for (const tag of order) {
      for (const fn of _.get(this.topics, [obj.topic, obj.method, tag], [])) {
        // log({ result, fn })
        result = run(fn, result, ctx)
      }
    }
    // log('last', { result })
    return result
  }
  order(topic, method) {
    return _.get(this.sets, [topic, method], []).slice()
  }
  reorder(topic, method, fn = _.noop) {
    let order = this.order(topic, method)
    let reorder
    if (_.isArr(fn))
      reorder = fn
    else if (_.isFn(fn))
      reorder = fn(order)
    if (reorder)
      order = reorder
    _.set(this.sets, [topic, method], order)
  }
  remove(topic, method, tags) {
    tags = _.toArr(tags)
    return this.reorder(topic, method, order => {
      log('remove', tags, 'from', order)
      return order
    })
  }
  ingest(Class) {
    let methods = _.getMethods(Class.prototype)
    let sort = {}
    for (let method of methods) {
      if (!method.startsWith('$'))
        continue
      let fn = Class.prototype[method]
      method = method.slice(1)
      let parts = method.split('_')
      if (parts.length === 2) {
        let [topic, tag] = parts
        let method = topic
        this.use([topic, method].join('_'), tag, fn)
      } else if (parts.length === 3) {
        if (_.isNum(_.last(parts))) {
          let [topic, meth, index] = parts
          _.set(sort, [topic, meth, index], method)
        } else {
          let [topic, meth, tag] = parts
          this.use([topic, meth].join('_'), tag, fn)
        }
      } else if (parts.length === 4) {
        log('handle 4 parts')
      }
    }
    for (let topic in sort) {
      for (let tag in sort[topic]) {
        let arr = sort[topic][tag]
        for (let name of arr) {
          if (!name)
            continue
          name = '$' + name
          let fn = Class.prototype[name]
          this.use([topic, topic].join('_'), tag, fn)
        }
      }
    }
    if (Class.prototype.install)
      Class.prototype.install.call(this.ctx || this, this)
    this.freeze()
  }
}

module.exports = Mu
