const { _, log } = require('./basd')

class Router {
  constructor(opts = {}) {
    this.routes = {}
    _.objProp(this, 'ctx', opts.ctx)
    const routes = _.omit(opts, ['ctx', 'prefix'])
    this.bulk(routes, opts.prefix, this.ctx)
  }
  receive(route, handler, ctx) {
    route = this.getRoute(route)
    ctx = ctx || this.ctx
    if (ctx)
      handler = handler.bind(ctx)
    _.set(this.routes, route, handler)
  }
  bulk(routes = {}, prefix, ctx) {
    if (prefix) {
      prefix = this.getRoute(prefix)
      const tmp = {}
      _.set(tmp, prefix, routes)
      routes = tmp
    }
    const flat = _.flatObj(routes)
    for (let route in flat)
      this.receive(route, flat[route], ctx)
  }
  ingest(obj, prefix, ctx, methodPrefix = 'on') {
    const routes = {}
    ctx = ctx || obj
    const fields = _.getMethods(obj)
    const matcher = field => field.startsWith(methodPrefix) && field.length > methodPrefix.length
    for (let field of fields) {
      if (matcher(field)) {
        // const handler = obj.constructor.prototype[field]
        const handler = obj[field]
        field = field.slice(methodPrefix.length)
        field = _.lc(field[0]) + field.slice(1)
        routes[_.kebabCase(field)] = handler
        routes[field] = handler
      }
    }
    this.bulk(routes, prefix, ctx)
  }
  getRoute(key) {
    if (_.isArr(key))
      key = key.join('.')
    if (key[0] === '/')
      key = key.slice(1)
    if (key[key.length - 1] === '/')
      key = key.slice(0, -1)
    return key.split('/').join('.')
  }
  _getRoute(route) {
    route = this.getRoute(route)
    return _.get(this.routes, route)
  }
  hasRoute(route) {
    return !!this._getRoute(route)
  }
  run(route, params, ...args) {
    const fn = this._getRoute(route)
    if (!_.isFn(fn)) {
      log(`>>> route "${route}" doesn't exist`)
      return Promise.resolve(null)
    }
    return _.respond(fn(params, ...args))
  }
}

module.exports = Router
