## Modules

<dl>
<dt><a href="#module_Base">Base</a></dt>
<dd><p>This file contains a collection of utility functions that are 
reused throughout the application. These functions include a variety of type 
checkers, conversion tools, and other utilities that aid in data manipulation 
and validation. This utility library is designed to be environment-agnostic, 
providing compatibility with both browser and Node.js environments.</p>
</dd>
<dt><a href="#module_Debug">Debug</a></dt>
<dd><p>This is a console log utility which supports different log 
levels (debug, info, warn, error, log). It provides enhanced logging 
capabilities with features such as color-coding, labeling, source tracking 
and time-stamping. It is a part of the <code>basd</code> foundational library which</p>
</dd>
<dt><a href="#module_Evented">Evented</a></dt>
<dd><p>Provides an event emitter mixin class for applications.</p>
<p>This module exports a mixin class <code>Evented</code> for creating objects with 
capabilities of event-driven behavior. It is part of the <code>basd</code> project 
and heavily relies on other foundational parts of the project.</p>
<p>The <code>Evented</code> class and its mixin function are exported from the module.</p>
</dd>
<dt><a href="#module_Heap">Heap</a></dt>
<dd><p>This file defines the Heap and QueueItem classes, used to
implement a heap data structure for use within the @basd/base submodule.
This heap implementation is a key part of the @basd foundational library.</p>
</dd>
<dt><a href="#module_Temporal">Temporal</a></dt>
<dd><p>This module, Temporal, is an event simulator for network activity. 
It can simulate different network events such as intervals, timeouts, and immediate
executions, that are crucial in various networking scenarios.</p>
</dd>
<dt><a href="#module_Timely">Timely</a></dt>
<dd><p>Defines Timer and Timely classes to create and manage 
timer events with a variety of options.</p>
</dd>
<dt><a href="#module_Timestamp">Timestamp</a></dt>
<dd><p>Timestamp - A module that generates universally unique hex string 
timestamps. The timestamps generated are unique across instances and even on 
the same millisecond. This module is part of the <code>basd</code> library, which is a 
foundational lib for many other projects.</p>
</dd>
</dl>

<a name="module_Base"></a>

## Base
This file contains a collection of utility functions that are 
reused throughout the application. These functions include a variety of type 
checkers, conversion tools, and other utilities that aid in data manipulation 
and validation. This utility library is designed to be environment-agnostic, 
providing compatibility with both browser and Node.js environments.

**Requires**: <code>module:lodash</code>, <code>module:bs58</code>, <code>module:lexicographic-integer</code>  
**See**: [Optional external link related to this file](http://example.com)  
**Version**: 0.0.1  
**Author**: Basedwon  

* [Base](#module_Base)
    * [~log](#module_Base..log) : <code>function</code>
    * [~print](#module_Base..print) : <code>function</code>
    * [~isNumeric](#module_Base..isNumeric) ⇒ <code>boolean</code>
    * [~toNumber](#module_Base..toNumber) ⇒ <code>number</code>
    * [~isDate](#module_Base..isDate) ⇒ <code>boolean</code>
    * [~isBs58](#module_Base..isBs58) ⇒ <code>boolean</code>
    * [~isBool](#module_Base..isBool) ⇒ <code>boolean</code>
    * [~toBoolean](#module_Base..toBoolean) ⇒ <code>boolean</code>
    * [~isObj](#module_Base..isObj) ⇒ <code>boolean</code>
    * [~toArray](#module_Base..toArray) ⇒ <code>Array</code>
    * [~isBinary](#module_Base..isBinary) ⇒ <code>boolean</code>
    * [~toString](#module_Base..toString) ⇒ <code>string</code> \| <code>any</code>
    * [~toBuffer](#module_Base..toBuffer) ⇒ <code>Uint8Array</code> \| <code>Buffer</code>
    * [~truthy](#module_Base..truthy) ⇒ <code>boolean</code>
    * [~falsey](#module_Base..falsey) ⇒ <code>boolean</code>
    * [~noop](#module_Base..noop)
    * [~isFn](#module_Base..isFn) ⇒ <code>boolean</code>
    * [~odd](#module_Base..odd) ⇒ <code>boolean</code>
    * [~even](#module_Base..even) ⇒ <code>boolean</code>
    * [~isBrowser](#module_Base..isBrowser) : <code>boolean</code>
    * [~isNode](#module_Base..isNode) : <code>boolean</code>
    * [~isMobile](#module_Base..isMobile) : <code>boolean</code>
    * [~URL](#module_Base..URL) : <code>URL</code>
    * [~isUrl](#module_Base..isUrl) ⇒ <code>boolean</code>
    * [~ucf](#module_Base..ucf) ⇒ <code>string</code>
    * [~uc](#module_Base..uc) ⇒ <code>string</code>
    * [~lc](#module_Base..lc) ⇒ <code>string</code>
    * [~pad](#module_Base..pad) ⇒ <code>string</code>
    * [~unpad](#module_Base..unpad) ⇒ <code>string</code>
    * [~sleep](#module_Base..sleep) ⇒ <code>Promise.&lt;void&gt;</code>
    * [~objProp](#module_Base..objProp) ⇒ <code>Object</code>
    * [~proxyGet](#module_Base..proxyGet) ⇒ <code>Proxy</code>
    * [~proxyNew](#module_Base..proxyNew) ⇒ <code>Proxy</code>
    * [~getMethods](#module_Base..getMethods) ⇒ <code>Array.&lt;string&gt;</code>
    * [~respond](#module_Base..respond) ⇒ <code>Promise.&lt;any&gt;</code>
    * [~encode](#module_Base..encode) ⇒ <code>string</code>
    * [~decode](#module_Base..decode) ⇒ <code>Buffer</code>
    * [~str2ms](#module_Base..str2ms) ⇒ <code>number</code>
    * [~sortChars](#module_Base..sortChars) ⇒ <code>string</code>

<a name="module_Base..log"></a>

### Base~log : <code>function</code>
Logs the data.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  

| Param | Type | Description |
| --- | --- | --- |
| ...args | <code>any</code> | Data to log. |

<a name="module_Base..print"></a>

### Base~print : <code>function</code>
Prints the data using the log function.
node version uses util to show all

**Kind**: inner constant of [<code>Base</code>](#module_Base)  

| Param | Type | Description |
| --- | --- | --- |
| ...args | <code>any</code> | Data to print. |

<a name="module_Base..isNumeric"></a>

### Base~isNumeric ⇒ <code>boolean</code>
Checks if a value is numeric.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the value is numeric.  

| Param | Type | Description |
| --- | --- | --- |
| num | <code>any</code> | Value to check. |

<a name="module_Base..toNumber"></a>

### Base~toNumber ⇒ <code>number</code>
Converts a value to a number.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>number</code> - The converted number.  

| Param | Type | Description |
| --- | --- | --- |
| num | <code>any</code> | Value to convert. |
| deci | <code>number</code> | Number of decimal places. |

<a name="module_Base..isDate"></a>

### Base~isDate ⇒ <code>boolean</code>
Checks if a value is a valid date.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the value is a valid date.  

| Param | Type | Description |
| --- | --- | --- |
| date | <code>any</code> | Value to check. |

<a name="module_Base..isBs58"></a>

### Base~isBs58 ⇒ <code>boolean</code>
Checks if a string is a valid Base58 string.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the string is Base58.  

| Param | Type | Description |
| --- | --- | --- |
| str | <code>string</code> | String to check. |

<a name="module_Base..isBool"></a>

### Base~isBool ⇒ <code>boolean</code>
Checks if a value is a boolean or "true" / "false" string.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the value is a boolean or "true" / "false" string.  

| Param | Type | Description |
| --- | --- | --- |
| bool | <code>any</code> | Value to check. |

<a name="module_Base..toBoolean"></a>

### Base~toBoolean ⇒ <code>boolean</code>
Converts a value to a boolean.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - The converted boolean.  

| Param | Type | Description |
| --- | --- | --- |
| bool | <code>any</code> | Value to convert. |

<a name="module_Base..isObj"></a>

### Base~isObj ⇒ <code>boolean</code>
Checks if a value is an object, excluding arrays, functions, and binary data.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the value is an object.  

| Param | Type | Description |
| --- | --- | --- |
| input | <code>any</code> | Value to check. |

<a name="module_Base..toArray"></a>

### Base~toArray ⇒ <code>Array</code>
Converts a value to an array. If the value is null or undefined, returns an empty array.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>Array</code> - The converted array.  

| Param | Type | Description |
| --- | --- | --- |
| obj | <code>any</code> | Value to convert. |

<a name="module_Base..isBinary"></a>

### Base~isBinary ⇒ <code>boolean</code>
Checks if a value is binary data.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the value is binary data.  

| Param | Type | Description |
| --- | --- | --- |
| value | <code>any</code> | Value to check. |

<a name="module_Base..toString"></a>

### Base~toString ⇒ <code>string</code> \| <code>any</code>
Converts a value to a string. If the value cannot be converted, it returns the original value.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>string</code> \| <code>any</code> - The converted string or the original value if conversion failed.  

| Param | Type | Description |
| --- | --- | --- |
| obj | <code>any</code> | Value to convert. |

<a name="module_Base..toBuffer"></a>

### Base~toBuffer ⇒ <code>Uint8Array</code> \| <code>Buffer</code>
Transforms any data into a buffer.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>Uint8Array</code> \| <code>Buffer</code> - The transformed buffer.  
**Throws**:

- <code>Error</code> If data type is unsupported.


| Param | Type | Description |
| --- | --- | --- |
| data | <code>any</code> | Data to transform into a buffer. |
| [encoding] | <code>string</code> | The encoding to use if not a browser environment. |

<a name="module_Base..truthy"></a>

### Base~truthy ⇒ <code>boolean</code>
Checks if a value is a truthy boolean.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the value is a truthy boolean.  

| Param | Type | Description |
| --- | --- | --- |
| obj | <code>any</code> | Value to check. |

<a name="module_Base..falsey"></a>

### Base~falsey ⇒ <code>boolean</code>
Checks if a value is a falsy boolean.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the value is a falsy boolean.  

| Param | Type | Description |
| --- | --- | --- |
| obj | <code>any</code> | Value to check. |

<a name="module_Base..noop"></a>

### Base~noop
A no-operation function that does nothing.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
<a name="module_Base..isFn"></a>

### Base~isFn ⇒ <code>boolean</code>
Checks if a value is a function.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the value is a function.  

| Param | Type | Description |
| --- | --- | --- |
| fn | <code>any</code> | Value to check. |

<a name="module_Base..odd"></a>

### Base~odd ⇒ <code>boolean</code>
Checks if a number is odd.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the number is odd.  

| Param | Type | Description |
| --- | --- | --- |
| num | <code>number</code> | Number to check. |

<a name="module_Base..even"></a>

### Base~even ⇒ <code>boolean</code>
Checks if a number is even.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the number is even.  

| Param | Type | Description |
| --- | --- | --- |
| num | <code>number</code> | Number to check. |

<a name="module_Base..isBrowser"></a>

### Base~isBrowser : <code>boolean</code>
Detects if the runtime environment is a browser.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
<a name="module_Base..isNode"></a>

### Base~isNode : <code>boolean</code>
Detects if the runtime environment is Node.js.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
<a name="module_Base..isMobile"></a>

### Base~isMobile : <code>boolean</code>
Detects if the device is a mobile device.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
<a name="module_Base..URL"></a>

### Base~URL : <code>URL</code>
Sets the URL object based on the runtime environment.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
<a name="module_Base..isUrl"></a>

### Base~isUrl ⇒ <code>boolean</code>
Checks if a string is a valid URL.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>boolean</code> - Whether the string is a valid URL.  

| Param | Type | Description |
| --- | --- | --- |
| url | <code>string</code> | String to check. |

<a name="module_Base..ucf"></a>

### Base~ucf ⇒ <code>string</code>
Capitalizes the first character of a string.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>string</code> - The capitalized string.  

| Param | Type | Description |
| --- | --- | --- |
| str | <code>string</code> | String to capitalize. |

<a name="module_Base..uc"></a>

### Base~uc ⇒ <code>string</code>
Converts a string to upper case.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>string</code> - The converted string.  

| Param | Type | Description |
| --- | --- | --- |
| str | <code>string</code> | String to convert. |

<a name="module_Base..lc"></a>

### Base~lc ⇒ <code>string</code>
Converts a string to lower case.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>string</code> - The converted string.  

| Param | Type | Description |
| --- | --- | --- |
| str | <code>string</code> | String to convert. |

<a name="module_Base..pad"></a>

### Base~pad ⇒ <code>string</code>
Pads a string from the start with spaces until it reaches the specified length.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>string</code> - The padded string.  

| Param | Type | Description |
| --- | --- | --- |
| str | <code>string</code> | String to pad. |
| len | <code>number</code> | Target length. |

<a name="module_Base..unpad"></a>

### Base~unpad ⇒ <code>string</code>
Removes leading white space from a string.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>string</code> - The trimmed string.  

| Param | Type | Description |
| --- | --- | --- |
| str | <code>string</code> | String to trim. |

<a name="module_Base..sleep"></a>

### Base~sleep ⇒ <code>Promise.&lt;void&gt;</code>
Resolves a promise after a delay.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>Promise.&lt;void&gt;</code> - A promise that resolves after the specified delay.  

| Param | Type | Default | Description |
| --- | --- | --- | --- |
| [ms] | <code>number</code> \| <code>string</code> | <code>0</code> | Delay in milliseconds or a string representing time. |

<a name="module_Base..objProp"></a>

### Base~objProp ⇒ <code>Object</code>
Defines a new property directly on an object, or modifies an existing property 
on an object, and returns the object.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>Object</code> - The object on which the property was defined.  

| Param | Type | Default | Description |
| --- | --- | --- | --- |
| obj | <code>Object</code> |  | The object on which to define or modify the property. |
| key | <code>string</code> \| <code>Symbol</code> |  | The name or Symbol of the property to be defined or modified. |
| value | <code>any</code> |  | The value to set the property to. |
| [opts] | <code>Object</code> | <code>{}</code> | The optional attributes for the property descriptor. |

<a name="module_Base..proxyGet"></a>

### Base~proxyGet ⇒ <code>Proxy</code>
Creates a Proxy for an object to customize the behavior of fundamental operations (e.g., property lookup, assignment, enumeration, function invocation, etc).

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>Proxy</code> - The Proxy for the object.  

| Param | Type | Description |
| --- | --- | --- |
| obj | <code>Object</code> | The target object for the Proxy. |
| callback | <code>function</code> | The handler callback for the get trap. |

<a name="module_Base..proxyNew"></a>

### Base~proxyNew ⇒ <code>Proxy</code>
Creates a Proxy for an object to customize its instantiation behavior.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>Proxy</code> - The Proxy for the object.  

| Param | Type | Description |
| --- | --- | --- |
| obj | <code>Object</code> | The target object for the Proxy. |

<a name="module_Base..getMethods"></a>

### Base~getMethods ⇒ <code>Array.&lt;string&gt;</code>
Returns the names of all methods of an object.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>Array.&lt;string&gt;</code> - The method names.  

| Param | Type | Default | Description |
| --- | --- | --- | --- |
| obj | <code>Object</code> |  | The object to get methods from. |
| [ignore] | <code>boolean</code> \| <code>Array.&lt;string&gt;</code> |  | Whether to ignore built-in methods or an array of method names to ignore. |
| [underscore] | <code>boolean</code> | <code>false</code> | Whether to include methods that start with an underscore. |
| [getters] | <code>boolean</code> | <code>false</code> | Whether to include getter methods. |

<a name="module_Base..respond"></a>

### Base~respond ⇒ <code>Promise.&lt;any&gt;</code>
Wraps a value in a promise if it is not already a promise, allowing it to be used as a promise.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>Promise.&lt;any&gt;</code> - The promise.  

| Param | Type | Description |
| --- | --- | --- |
| res | <code>any</code> | The value or promise. |

<a name="module_Base..encode"></a>

### Base~encode ⇒ <code>string</code>
Encodes a buffer to a Base58 or hex string.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>string</code> - The encoded string.  

| Param | Type | Description |
| --- | --- | --- |
| buff | <code>Buffer</code> | Buffer to encode. |
| [hex] | <code>boolean</code> | Whether to encode to hex instead of Base58. |

<a name="module_Base..decode"></a>

### Base~decode ⇒ <code>Buffer</code>
Decodes a Base58 or hex string into a buffer.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>Buffer</code> - The decoded buffer.  
**Throws**:

- <code>Error</code> If the input string is not a string.


| Param | Type | Default | Description |
| --- | --- | --- | --- |
| str | <code>string</code> |  | String to decode. |
| [toBuffer] | <code>boolean</code> \| <code>string</code> | <code>true</code> | Whether to convert the result to a buffer. If 'hex', the result is converted to a hex buffer. |

<a name="module_Base..str2ms"></a>

### Base~str2ms ⇒ <code>number</code>
Converts a string representing time to milliseconds.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>number</code> - The converted milliseconds.  

| Param | Type | Default | Description |
| --- | --- | --- | --- |
| str | <code>string</code> |  | String to convert. |
| [obj] | <code>Object</code> | <code>{}</code> | Object to populate with conversion details. |

<a name="module_Base..sortChars"></a>

### Base~sortChars ⇒ <code>string</code>
Sorts the characters of a string or a JSON stringified object.

**Kind**: inner constant of [<code>Base</code>](#module_Base)  
**Returns**: <code>string</code> - The string with sorted characters.  

| Param | Type | Description |
| --- | --- | --- |
| data | <code>string</code> \| <code>Object</code> | String or object to sort characters of. |

<a name="module_Debug"></a>

## Debug
This is a console log utility which supports different log 
levels (debug, info, warn, error, log). It provides enhanced logging 
capabilities with features such as color-coding, labeling, source tracking 
and time-stamping. It is a part of the `basd` foundational library which


* [Debug](#module_Debug)
    * [~debug](#module_Debug..debug)
        * [new debug(level, ...args)](#new_module_Debug..debug_new)
        * _instance_
            * [.levels](#module_Debug..debug+levels)
            * [.colorMap](#module_Debug..debug+colorMap)
            * [.colors](#module_Debug..debug+colors)
            * [.level](#module_Debug..debug+level)
            * [.getColor(level)](#module_Debug..debug+getColor) ⇒ <code>string</code>
            * [.getColorNum(level)](#module_Debug..debug+getColorNum) ⇒ <code>number</code>
            * [.getCaller()](#module_Debug..debug+getCaller) ⇒ <code>string</code>
            * [.getTime()](#module_Debug..debug+getTime) ⇒ <code>string</code>
            * [.parse()](#module_Debug..debug+parse)
            * [.format()](#module_Debug..debug+format)
            * [.execute()](#module_Debug..debug+execute)
            * [.print()](#module_Debug..debug+print)
        * _static_
            * [.print(...args)](#module_Debug..debug.print) ⇒ <code>void</code>
            * [.debug(...args)](#module_Debug..debug.debug) ⇒ <code>void</code>
            * [.info(...args)](#module_Debug..debug.info) ⇒ <code>void</code>
            * [.warn(...args)](#module_Debug..debug.warn) ⇒ <code>debug</code>
            * [.error(...args)](#module_Debug..debug.error) ⇒ <code>debug</code>
            * [.log(...args)](#module_Debug..debug.log) ⇒ <code>void</code>
            * [.set(key, value)](#module_Debug..debug.set) ⇒ <code>debug</code>
            * [.show(key, [value])](#module_Debug..debug.show) ⇒ <code>debug</code>
            * [.hide(key, [value])](#module_Debug..debug.hide) ⇒ <code>debug</code>
            * [.showError(...args)](#module_Debug..debug.showError) ⇒ <code>debug</code>
            * [.return(result, ...args)](#module_Debug..debug.return) ⇒ <code>any</code>
    * [~apply(target, receiver, args)](#module_Debug..apply) ⇒ <code>void</code>

<a name="module_Debug..debug"></a>

### Debug~debug
A utility class for improved console logging with support for
debug, info, warn, error and log levels.

**Kind**: inner class of [<code>Debug</code>](#module_Debug)  

* [~debug](#module_Debug..debug)
    * [new debug(level, ...args)](#new_module_Debug..debug_new)
    * _instance_
        * [.levels](#module_Debug..debug+levels)
        * [.colorMap](#module_Debug..debug+colorMap)
        * [.colors](#module_Debug..debug+colors)
        * [.level](#module_Debug..debug+level)
        * [.getColor(level)](#module_Debug..debug+getColor) ⇒ <code>string</code>
        * [.getColorNum(level)](#module_Debug..debug+getColorNum) ⇒ <code>number</code>
        * [.getCaller()](#module_Debug..debug+getCaller) ⇒ <code>string</code>
        * [.getTime()](#module_Debug..debug+getTime) ⇒ <code>string</code>
        * [.parse()](#module_Debug..debug+parse)
        * [.format()](#module_Debug..debug+format)
        * [.execute()](#module_Debug..debug+execute)
        * [.print()](#module_Debug..debug+print)
    * _static_
        * [.print(...args)](#module_Debug..debug.print) ⇒ <code>void</code>
        * [.debug(...args)](#module_Debug..debug.debug) ⇒ <code>void</code>
        * [.info(...args)](#module_Debug..debug.info) ⇒ <code>void</code>
        * [.warn(...args)](#module_Debug..debug.warn) ⇒ <code>debug</code>
        * [.error(...args)](#module_Debug..debug.error) ⇒ <code>debug</code>
        * [.log(...args)](#module_Debug..debug.log) ⇒ <code>void</code>
        * [.set(key, value)](#module_Debug..debug.set) ⇒ <code>debug</code>
        * [.show(key, [value])](#module_Debug..debug.show) ⇒ <code>debug</code>
        * [.hide(key, [value])](#module_Debug..debug.hide) ⇒ <code>debug</code>
        * [.showError(...args)](#module_Debug..debug.showError) ⇒ <code>debug</code>
        * [.return(result, ...args)](#module_Debug..debug.return) ⇒ <code>any</code>

<a name="new_module_Debug..debug_new"></a>

#### new debug(level, ...args)
Initializes a new debug instance.


| Param | Type | Description |
| --- | --- | --- |
| level | <code>string</code> | The logging level. |
| ...args | <code>any</code> | Additional arguments for the log. |

<a name="module_Debug..debug+levels"></a>

#### debug.levels
Logging levels with corresponding numbers.

**Kind**: instance property of [<code>debug</code>](#module_Debug..debug)  
<a name="module_Debug..debug+colorMap"></a>

#### debug.colorMap
Color mapping for different log levels.

**Kind**: instance property of [<code>debug</code>](#module_Debug..debug)  
<a name="module_Debug..debug+colors"></a>

#### debug.colors
Corresponding color codes for terminal.

**Kind**: instance property of [<code>debug</code>](#module_Debug..debug)  
<a name="module_Debug..debug+level"></a>

#### debug.level
The current log level to be used.

**Kind**: instance property of [<code>debug</code>](#module_Debug..debug)  
<a name="module_Debug..debug+getColor"></a>

#### debug.getColor(level) ⇒ <code>string</code>
Returns the color for the given log level.

**Kind**: instance method of [<code>debug</code>](#module_Debug..debug)  
**Returns**: <code>string</code> - - The color for the log level.  

| Param | Type | Description |
| --- | --- | --- |
| level | <code>string</code> | The log level. |

<a name="module_Debug..debug+getColorNum"></a>

#### debug.getColorNum(level) ⇒ <code>number</code>
Returns the color code for the given log level.

**Kind**: instance method of [<code>debug</code>](#module_Debug..debug)  
**Returns**: <code>number</code> - - The color code for the log level.  

| Param | Type | Description |
| --- | --- | --- |
| level | <code>string</code> | The log level. |

<a name="module_Debug..debug+getCaller"></a>

#### debug.getCaller() ⇒ <code>string</code>
Returns the caller's information (method, file path, line number, and column number).

**Kind**: instance method of [<code>debug</code>](#module_Debug..debug)  
**Returns**: <code>string</code> - - The caller's information.  
<a name="module_Debug..debug+getTime"></a>

#### debug.getTime() ⇒ <code>string</code>
Returns the current time in the format of the local time string.

**Kind**: instance method of [<code>debug</code>](#module_Debug..debug)  
**Returns**: <code>string</code> - - The current local time string.  
<a name="module_Debug..debug+parse"></a>

#### debug.parse()
Parses the log level and arguments.

**Kind**: instance method of [<code>debug</code>](#module_Debug..debug)  
<a name="module_Debug..debug+format"></a>

#### debug.format()
Formats the log message and adds it to the output.

**Kind**: instance method of [<code>debug</code>](#module_Debug..debug)  
<a name="module_Debug..debug+execute"></a>

#### debug.execute()
Executes the log command.

**Kind**: instance method of [<code>debug</code>](#module_Debug..debug)  
<a name="module_Debug..debug+print"></a>

#### debug.print()
Performs parsing, formatting and execution of log message.

**Kind**: instance method of [<code>debug</code>](#module_Debug..debug)  
<a name="module_Debug..debug.print"></a>

#### debug.print(...args) ⇒ <code>void</code>
Parses, formats and logs a message using the debug instance.

**Kind**: static method of [<code>debug</code>](#module_Debug..debug)  

| Param | Type | Description |
| --- | --- | --- |
| ...args | <code>any</code> | Arguments for the log. |

<a name="module_Debug..debug.debug"></a>

#### debug.debug(...args) ⇒ <code>void</code>
Parses, formats and logs a debug message.

**Kind**: static method of [<code>debug</code>](#module_Debug..debug)  

| Param | Type | Description |
| --- | --- | --- |
| ...args | <code>any</code> | Arguments for the log. |

<a name="module_Debug..debug.info"></a>

#### debug.info(...args) ⇒ <code>void</code>
Parses, formats and logs an info message.

**Kind**: static method of [<code>debug</code>](#module_Debug..debug)  

| Param | Type | Description |
| --- | --- | --- |
| ...args | <code>any</code> | Arguments for the log. |

<a name="module_Debug..debug.warn"></a>

#### debug.warn(...args) ⇒ <code>debug</code>
Parses, formats and logs a warning message.

**Kind**: static method of [<code>debug</code>](#module_Debug..debug)  
**Returns**: <code>debug</code> - - The debug class.  

| Param | Type | Description |
| --- | --- | --- |
| ...args | <code>any</code> | Arguments for the log. |

<a name="module_Debug..debug.error"></a>

#### debug.error(...args) ⇒ <code>debug</code>
Parses, formats and logs an error message. 
If the first argument is a string, it will be converted to an Error object.

**Kind**: static method of [<code>debug</code>](#module_Debug..debug)  
**Returns**: <code>debug</code> - - The debug class.  

| Param | Type | Description |
| --- | --- | --- |
| ...args | <code>any</code> | Arguments for the log. |

<a name="module_Debug..debug.log"></a>

#### debug.log(...args) ⇒ <code>void</code>
Parses, formats and logs a generic log message.

**Kind**: static method of [<code>debug</code>](#module_Debug..debug)  

| Param | Type | Description |
| --- | --- | --- |
| ...args | <code>any</code> | Arguments for the log. |

<a name="module_Debug..debug.set"></a>

#### debug.set(key, value) ⇒ <code>debug</code>
Sets the value of a given key.

**Kind**: static method of [<code>debug</code>](#module_Debug..debug)  
**Returns**: <code>debug</code> - - The debug class.  

| Param | Type | Description |
| --- | --- | --- |
| key | <code>string</code> | The key to set. |
| value | <code>any</code> | The value to set. |

<a name="module_Debug..debug.show"></a>

#### debug.show(key, [value]) ⇒ <code>debug</code>
Shows a given key.

**Kind**: static method of [<code>debug</code>](#module_Debug..debug)  
**Returns**: <code>debug</code> - - The debug class.  

| Param | Type | Default | Description |
| --- | --- | --- | --- |
| key | <code>string</code> |  | The key to show. |
| [value] | <code>boolean</code> | <code>true</code> | The value to set. |

<a name="module_Debug..debug.hide"></a>

#### debug.hide(key, [value]) ⇒ <code>debug</code>
Hides a given key.

**Kind**: static method of [<code>debug</code>](#module_Debug..debug)  
**Returns**: <code>debug</code> - - The debug class.  

| Param | Type | Default | Description |
| --- | --- | --- | --- |
| key | <code>string</code> |  | The key to hide. |
| [value] | <code>boolean</code> | <code>false</code> | The value to set. |

<a name="module_Debug..debug.showError"></a>

#### debug.showError(...args) ⇒ <code>debug</code>
Shows an error message.

**Kind**: static method of [<code>debug</code>](#module_Debug..debug)  
**Returns**: <code>debug</code> - - The debug class.  

| Param | Type | Description |
| --- | --- | --- |
| ...args | <code>any</code> | Arguments for the error log. |

<a name="module_Debug..debug.return"></a>

#### debug.return(result, ...args) ⇒ <code>any</code>
Logs a message and returns the result.

**Kind**: static method of [<code>debug</code>](#module_Debug..debug)  
**Returns**: <code>any</code> - - The result.  

| Param | Type | Description |
| --- | --- | --- |
| result | <code>any</code> | The result to return. |
| ...args | <code>any</code> | Arguments for the log. |

<a name="module_Debug..apply"></a>

### Debug~apply(target, receiver, args) ⇒ <code>void</code>
Proxy handler for the `apply` trap.

**Kind**: inner method of [<code>Debug</code>](#module_Debug)  

| Param | Type | Description |
| --- | --- | --- |
| target | <code>debug</code> | The target debug instance. |
| receiver | <code>any</code> | The receiver argument (usually the proxy itself). |
| args | <code>Array.&lt;any&gt;</code> | Arguments for the call. |

<a name="module_Evented"></a>

## Evented
Provides an event emitter mixin class for applications.

This module exports a mixin class `Evented` for creating objects with 
capabilities of event-driven behavior. It is part of the `basd` project 
and heavily relies on other foundational parts of the project.

The `Evented` class and its mixin function are exported from the module.


* [Evented](#module_Evented)
    * [~Evented](#module_Evented..Evented)
        * [new Evented(Class)](#new_module_Evented..Evented_new)
    * [~on(topic, handler)](#module_Evented..on) ⇒ <code>function</code>
    * [~onDefault(topic, handler)](#module_Evented..onDefault)
    * [~off(topic, handler)](#module_Evented..off)
    * [~once(topic, handler)](#module_Evented..once)
    * [~emit(topic, ...args)](#module_Evented..emit)
    * [~has(topic)](#module_Evented..has) ⇒ <code>boolean</code>
    * [~size(topic)](#module_Evented..size) ⇒ <code>number</code>
    * [~clear(topic)](#module_Evented..clear)

<a name="module_Evented..Evented"></a>

### Evented~Evented
**Kind**: inner class of [<code>Evented</code>](#module_Evented)  
<a name="new_module_Evented..Evented_new"></a>

#### new Evented(Class)
**Returns**: <code>Evented</code> - - A class extended with event capabilities.  

| Param | Type | Description |
| --- | --- | --- |
| Class | <code>Class</code> | The class to extend with event capabilities. |

<a name="module_Evented..on"></a>

### Evented~on(topic, handler) ⇒ <code>function</code>
Registers a handler function for a specific topic.

**Kind**: inner method of [<code>Evented</code>](#module_Evented)  
**Returns**: <code>function</code> - - The off function, pre-bound with topic and handler.  

| Param | Type | Description |
| --- | --- | --- |
| topic | <code>string</code> | The event topic. |
| handler | <code>function</code> | The handler function for the event. |

<a name="module_Evented..onDefault"></a>

### Evented~onDefault(topic, handler)
Registers a default handler function for a specific topic.

**Kind**: inner method of [<code>Evented</code>](#module_Evented)  

| Param | Type | Description |
| --- | --- | --- |
| topic | <code>string</code> | The event topic. |
| handler | <code>function</code> | The handler function for the event. |

<a name="module_Evented..off"></a>

### Evented~off(topic, handler)
Deregisters a handler function from a specific topic.

**Kind**: inner method of [<code>Evented</code>](#module_Evented)  

| Param | Type | Description |
| --- | --- | --- |
| topic | <code>string</code> | The event topic. |
| handler | <code>function</code> | The handler function to remove. |

<a name="module_Evented..once"></a>

### Evented~once(topic, handler)
Registers a handler function for a specific topic for once.

**Kind**: inner method of [<code>Evented</code>](#module_Evented)  

| Param | Type | Description |
| --- | --- | --- |
| topic | <code>string</code> | The event topic. |
| handler | <code>function</code> | The handler function for the event. |

<a name="module_Evented..emit"></a>

### Evented~emit(topic, ...args)
Emits an event on a specific topic, executing all handlers.

**Kind**: inner method of [<code>Evented</code>](#module_Evented)  

| Param | Type | Description |
| --- | --- | --- |
| topic | <code>string</code> | The event topic. |
| ...args | <code>\*</code> | The arguments to pass to the handlers. |

<a name="module_Evented..has"></a>

### Evented~has(topic) ⇒ <code>boolean</code>
Checks if there are any handlers for a specific topic.

**Kind**: inner method of [<code>Evented</code>](#module_Evented)  
**Returns**: <code>boolean</code> - - Whether any handlers exist for the given topic.  

| Param | Type | Description |
| --- | --- | --- |
| topic | <code>string</code> | The event topic. |

<a name="module_Evented..size"></a>

### Evented~size(topic) ⇒ <code>number</code>
Returns the number of handlers for a specific topic.

**Kind**: inner method of [<code>Evented</code>](#module_Evented)  
**Returns**: <code>number</code> - - The number of handlers for the given topic.  

| Param | Type | Description |
| --- | --- | --- |
| topic | <code>string</code> | The event topic. |

<a name="module_Evented..clear"></a>

### Evented~clear(topic)
Clears all handlers for a specific topic.

**Kind**: inner method of [<code>Evented</code>](#module_Evented)  

| Param | Type | Description |
| --- | --- | --- |
| topic | <code>string</code> | The event topic. |

<a name="module_Heap"></a>

## Heap
This file defines the Heap and QueueItem classes, used to
implement a heap data structure for use within the @basd/base submodule.
This heap implementation is a key part of the @basd foundational library.


* [Heap](#module_Heap)
    * [~QueueItem](#module_Heap..QueueItem)
        * [new QueueItem(value, key)](#new_module_Heap..QueueItem_new)
        * [.getKey()](#module_Heap..QueueItem+getKey) ⇒ <code>number</code>
        * [.getValue()](#module_Heap..QueueItem+getValue) ⇒ <code>any</code>
        * [.clone()](#module_Heap..QueueItem+clone) ⇒ <code>QueueItem</code>
    * [~Heap](#module_Heap..Heap)
        * [new Heap(opts)](#new_module_Heap..Heap_new)
        * [.length](#module_Heap..Heap+length) ⇒ <code>number</code>
        * [.heapify(unorderedItems)](#module_Heap..Heap+heapify)
        * [.insert(value, key)](#module_Heap..Heap+insert)
        * [.remove()](#module_Heap..Heap+remove) ⇒ <code>any</code> \| <code>null</code>
        * [.peek()](#module_Heap..Heap+peek) ⇒ <code>any</code> \| <code>null</code>
        * [.peekKey()](#module_Heap..Heap+peekKey) ⇒ <code>number</code> \| <code>undefined</code>
        * [.isEmpty()](#module_Heap..Heap+isEmpty) ⇒ <code>boolean</code>
        * [.clear()](#module_Heap..Heap+clear)

<a name="module_Heap..QueueItem"></a>

### Heap~QueueItem
Represents an item in a heap.

**Kind**: inner class of [<code>Heap</code>](#module_Heap)  

* [~QueueItem](#module_Heap..QueueItem)
    * [new QueueItem(value, key)](#new_module_Heap..QueueItem_new)
    * [.getKey()](#module_Heap..QueueItem+getKey) ⇒ <code>number</code>
    * [.getValue()](#module_Heap..QueueItem+getValue) ⇒ <code>any</code>
    * [.clone()](#module_Heap..QueueItem+clone) ⇒ <code>QueueItem</code>

<a name="new_module_Heap..QueueItem_new"></a>

#### new QueueItem(value, key)
Constructs a QueueItem instance.


| Param | Type | Default | Description |
| --- | --- | --- | --- |
| value | <code>any</code> |  | The value of the item. |
| key | <code>number</code> | <code>1</code> | The sorting key of the item. |

<a name="module_Heap..QueueItem+getKey"></a>

#### queueItem.getKey() ⇒ <code>number</code>
Returns the key of the item.

**Kind**: instance method of [<code>QueueItem</code>](#module_Heap..QueueItem)  
**Returns**: <code>number</code> - The key of the item.  
<a name="module_Heap..QueueItem+getValue"></a>

#### queueItem.getValue() ⇒ <code>any</code>
Returns the value of the item.

**Kind**: instance method of [<code>QueueItem</code>](#module_Heap..QueueItem)  
**Returns**: <code>any</code> - The value of the item.  
<a name="module_Heap..QueueItem+clone"></a>

#### queueItem.clone() ⇒ <code>QueueItem</code>
Creates a new QueueItem instance with the same key and value as this one.

**Kind**: instance method of [<code>QueueItem</code>](#module_Heap..QueueItem)  
**Returns**: <code>QueueItem</code> - The cloned QueueItem.  
<a name="module_Heap..Heap"></a>

### Heap~Heap
Represents a heap data structure.

**Kind**: inner class of [<code>Heap</code>](#module_Heap)  

* [~Heap](#module_Heap..Heap)
    * [new Heap(opts)](#new_module_Heap..Heap_new)
    * [.length](#module_Heap..Heap+length) ⇒ <code>number</code>
    * [.heapify(unorderedItems)](#module_Heap..Heap+heapify)
    * [.insert(value, key)](#module_Heap..Heap+insert)
    * [.remove()](#module_Heap..Heap+remove) ⇒ <code>any</code> \| <code>null</code>
    * [.peek()](#module_Heap..Heap+peek) ⇒ <code>any</code> \| <code>null</code>
    * [.peekKey()](#module_Heap..Heap+peekKey) ⇒ <code>number</code> \| <code>undefined</code>
    * [.isEmpty()](#module_Heap..Heap+isEmpty) ⇒ <code>boolean</code>
    * [.clear()](#module_Heap..Heap+clear)

<a name="new_module_Heap..Heap_new"></a>

#### new Heap(opts)
Constructs a Heap instance.


| Param | Type | Description |
| --- | --- | --- |
| opts | <code>object</code> | The options for the heap, including initial items and a custom comparator function. |

<a name="module_Heap..Heap+length"></a>

#### heap.length ⇒ <code>number</code>
Returns the number of items in the heap.

**Kind**: instance property of [<code>Heap</code>](#module_Heap..Heap)  
**Returns**: <code>number</code> - The number of items in the heap.  
<a name="module_Heap..Heap+heapify"></a>

#### heap.heapify(unorderedItems)
Creates a valid heap from an unordered array of items in linear time.

**Kind**: instance method of [<code>Heap</code>](#module_Heap..Heap)  

| Param | Type | Description |
| --- | --- | --- |
| unorderedItems | <code>Array.&lt;{key: number, value: any}&gt;</code> | An array of objects, each with a 'key' for ordering and a 'value'. |

<a name="module_Heap..Heap+insert"></a>

#### heap.insert(value, key)
Inserts an item into the heap.

**Kind**: instance method of [<code>Heap</code>](#module_Heap..Heap)  

| Param | Type | Description |
| --- | --- | --- |
| value | <code>any</code> | The value of the item. |
| key | <code>number</code> | The key of the item. |

<a name="module_Heap..Heap+remove"></a>

#### heap.remove() ⇒ <code>any</code> \| <code>null</code>
Removes and returns the item at the top of the heap.

**Kind**: instance method of [<code>Heap</code>](#module_Heap..Heap)  
**Returns**: <code>any</code> \| <code>null</code> - The value of the removed item or null if the heap is empty.  
<a name="module_Heap..Heap+peek"></a>

#### heap.peek() ⇒ <code>any</code> \| <code>null</code>
Returns the value of the item at the top of the heap without removing it.

**Kind**: instance method of [<code>Heap</code>](#module_Heap..Heap)  
**Returns**: <code>any</code> \| <code>null</code> - The value of the top item or null if the heap is empty.  
<a name="module_Heap..Heap+peekKey"></a>

#### heap.peekKey() ⇒ <code>number</code> \| <code>undefined</code>
Returns the key of the item at the top of the heap.

**Kind**: instance method of [<code>Heap</code>](#module_Heap..Heap)  
**Returns**: <code>number</code> \| <code>undefined</code> - The key of the top item or undefined if the heap is empty.  
<a name="module_Heap..Heap+isEmpty"></a>

#### heap.isEmpty() ⇒ <code>boolean</code>
Checks if the heap is empty.

**Kind**: instance method of [<code>Heap</code>](#module_Heap..Heap)  
**Returns**: <code>boolean</code> - True if the heap is empty, false otherwise.  
<a name="module_Heap..Heap+clear"></a>

#### heap.clear()
Clears all items from the heap.

**Kind**: instance method of [<code>Heap</code>](#module_Heap..Heap)  
<a name="module_Temporal"></a>

## Temporal
This module, Temporal, is an event simulator for network activity. 
It can simulate different network events such as intervals, timeouts, and immediate
executions, that are crucial in various networking scenarios.


* [Temporal](#module_Temporal)
    * [~Temporal](#module_Temporal..Temporal)
        * [new Temporal(opts)](#new_module_Temporal..Temporal_new)
        * [.now()](#module_Temporal..Temporal+now) ⇒ <code>number</code>
        * [.start(speed, ms)](#module_Temporal..Temporal+start) ⇒ <code>Promise</code>
        * [.run(ms)](#module_Temporal..Temporal+run) ⇒ <code>Promise</code>
        * [.exec(fn, ctx, ...args)](#module_Temporal..Temporal+exec) ⇒ <code>Promise</code>
        * [.delay(delay, fn, ctx, ...args)](#module_Temporal..Temporal+delay) ⇒ <code>Promise</code>
        * [.tick(delay, fn, ctx, ...args)](#module_Temporal..Temporal+tick) ⇒ <code>Promise</code>
        * [.stop()](#module_Temporal..Temporal+stop)

<a name="module_Temporal..Temporal"></a>

### Temporal~Temporal
The Temporal class simulates the passage of time in a network environment.
It is equipped to handle different types of events such as intervals, timeouts, and 
immediate executions.

**Kind**: inner class of [<code>Temporal</code>](#module_Temporal)  
**Properties**

| Name | Type | Description |
| --- | --- | --- |
| began | <code>number</code> | The timestamp of when the Temporal instance started. |
| time | <code>number</code> | The current time in the Temporal instance. |
| maxrun | <code>number</code> | The maximum duration for the Temporal instance to run. |
| speed | <code>number</code> | The speed at which time passes in the Temporal instance. |
| running | <code>boolean</code> | Whether the Temporal instance is currently running. |
| _heap | <code>Heap</code> | A heap data structure to hold the events in a priority queue. |
| _timer | <code>number</code> | A reference to the setTimeout that's currently running. |


* [~Temporal](#module_Temporal..Temporal)
    * [new Temporal(opts)](#new_module_Temporal..Temporal_new)
    * [.now()](#module_Temporal..Temporal+now) ⇒ <code>number</code>
    * [.start(speed, ms)](#module_Temporal..Temporal+start) ⇒ <code>Promise</code>
    * [.run(ms)](#module_Temporal..Temporal+run) ⇒ <code>Promise</code>
    * [.exec(fn, ctx, ...args)](#module_Temporal..Temporal+exec) ⇒ <code>Promise</code>
    * [.delay(delay, fn, ctx, ...args)](#module_Temporal..Temporal+delay) ⇒ <code>Promise</code>
    * [.tick(delay, fn, ctx, ...args)](#module_Temporal..Temporal+tick) ⇒ <code>Promise</code>
    * [.stop()](#module_Temporal..Temporal+stop)

<a name="new_module_Temporal..Temporal_new"></a>

#### new Temporal(opts)

| Param | Type | Description |
| --- | --- | --- |
| opts | <code>Object</code> | The options to configure the Temporal instance. |

<a name="module_Temporal..Temporal+now"></a>

#### temporal.now() ⇒ <code>number</code>
Returns the current time in the Temporal instance.

**Kind**: instance method of [<code>Temporal</code>](#module_Temporal..Temporal)  
**Returns**: <code>number</code> - The current time in the Temporal instance.  
<a name="module_Temporal..Temporal+start"></a>

#### temporal.start(speed, ms) ⇒ <code>Promise</code>
Starts the Temporal instance.

**Kind**: instance method of [<code>Temporal</code>](#module_Temporal..Temporal)  
**Returns**: <code>Promise</code> - A promise that resolves when the Temporal instance starts.  

| Param | Type | Description |
| --- | --- | --- |
| speed | <code>number</code> | The speed at which time should pass. |
| ms | <code>string</code> \| <code>number</code> | The duration the Temporal instance should run. |

<a name="module_Temporal..Temporal+run"></a>

#### temporal.run(ms) ⇒ <code>Promise</code>
Runs the Temporal instance for a given duration.

**Kind**: instance method of [<code>Temporal</code>](#module_Temporal..Temporal)  
**Returns**: <code>Promise</code> - A promise that resolves when the Temporal instance starts running.  

| Param | Type | Description |
| --- | --- | --- |
| ms | <code>string</code> \| <code>number</code> | The duration the Temporal instance should run. |

<a name="module_Temporal..Temporal+exec"></a>

#### temporal.exec(fn, ctx, ...args) ⇒ <code>Promise</code>
Executes a function immediately in the Temporal instance.

**Kind**: instance method of [<code>Temporal</code>](#module_Temporal..Temporal)  
**Returns**: <code>Promise</code> - A promise that resolves when the function has been executed.  

| Param | Type | Description |
| --- | --- | --- |
| fn | <code>function</code> | The function to execute. |
| ctx | <code>Object</code> | The context to bind to the function. |
| ...args | <code>\*</code> | The arguments to pass to the function. |

<a name="module_Temporal..Temporal+delay"></a>

#### temporal.delay(delay, fn, ctx, ...args) ⇒ <code>Promise</code>
Executes a function after a specified delay in the Temporal instance.

**Kind**: instance method of [<code>Temporal</code>](#module_Temporal..Temporal)  
**Returns**: <code>Promise</code> - A promise that resolves when the function has been executed.  

| Param | Type | Description |
| --- | --- | --- |
| delay | <code>string</code> \| <code>number</code> | The delay after which the function should be executed. |
| fn | <code>function</code> | The function to execute. |
| ctx | <code>Object</code> | The context to bind to the function. |
| ...args | <code>\*</code> | The arguments to pass to the function. |

<a name="module_Temporal..Temporal+tick"></a>

#### temporal.tick(delay, fn, ctx, ...args) ⇒ <code>Promise</code>
Executes a function repeatedly, with a specified delay between executions, in the Temporal instance.

**Kind**: instance method of [<code>Temporal</code>](#module_Temporal..Temporal)  
**Returns**: <code>Promise</code> - A promise that resolves when the function has started being executed repeatedly.  

| Param | Type | Description |
| --- | --- | --- |
| delay | <code>string</code> \| <code>number</code> | The delay between executions of the function. |
| fn | <code>function</code> | The function to execute. |
| ctx | <code>Object</code> | The context to bind to the function. |
| ...args | <code>\*</code> | The arguments to pass to the function. |

<a name="module_Temporal..Temporal+stop"></a>

#### temporal.stop()
Stops the Temporal instance.

**Kind**: instance method of [<code>Temporal</code>](#module_Temporal..Temporal)  
<a name="module_Timely"></a>

## Timely
Defines Timer and Timely classes to create and manage 
timer events with a variety of options.

**Requires**: <code>module:./base</code>  

* [Timely](#module_Timely)
    * [~Timer](#module_Timely..Timer)
        * [new Timer(timer, clear, duration, fn)](#new_module_Timely..Timer_new)
        * [.remaining()](#module_Timely..Timer+remaining) ⇒ <code>number</code>
        * [.taken()](#module_Timely..Timer+taken) ⇒ <code>number</code>
    * [~Timely](#module_Timely..Timely)
        * [new Timely([opts], [ctx])](#new_module_Timely..Timely_new)
        * [.getFn(name, clear)](#module_Timely..Timely+getFn) ⇒ <code>function</code>
        * [.setTimeout(name, fn, time)](#module_Timely..Timely+setTimeout) ⇒ <code>this</code>
        * [.setInterval(name, fn, time)](#module_Timely..Timely+setInterval) ⇒ <code>this</code>
        * [.setImmediate(name, fn)](#module_Timely..Timely+setImmediate) ⇒ <code>this</code>
        * [.active(name)](#module_Timely..Timely+active) ⇒ <code>boolean</code>
        * [.clear(...args)](#module_Timely..Timely+clear) ⇒ <code>this</code>
        * [.adjust(name, time)](#module_Timely..Timely+adjust) ⇒ <code>this</code>
        * [.end()](#module_Timely..Timely+end) ⇒ <code>boolean</code>

<a name="module_Timely..Timer"></a>

### Timely~Timer
The Timer class wraps around timer functions and provides utility 
methods for time calculations.

**Kind**: inner class of [<code>Timely</code>](#module_Timely)  

* [~Timer](#module_Timely..Timer)
    * [new Timer(timer, clear, duration, fn)](#new_module_Timely..Timer_new)
    * [.remaining()](#module_Timely..Timer+remaining) ⇒ <code>number</code>
    * [.taken()](#module_Timely..Timer+taken) ⇒ <code>number</code>

<a name="new_module_Timely..Timer_new"></a>

#### new Timer(timer, clear, duration, fn)
Constructs a new Timer object.


| Param | Type | Description |
| --- | --- | --- |
| timer | <code>function</code> | Function to set the timer. |
| clear | <code>function</code> | Function to clear the timer. |
| duration | <code>number</code> | Timer duration in milliseconds. |
| fn | <code>function</code> | Function to be executed when timer finishes. |

<a name="module_Timely..Timer+remaining"></a>

#### timer.remaining() ⇒ <code>number</code>
Calculate the remaining time on the timer.

**Kind**: instance method of [<code>Timer</code>](#module_Timely..Timer)  
**Returns**: <code>number</code> - Remaining time in milliseconds.  
<a name="module_Timely..Timer+taken"></a>

#### timer.taken() ⇒ <code>number</code>
Calculate the elapsed time since the timer started.

**Kind**: instance method of [<code>Timer</code>](#module_Timely..Timer)  
**Returns**: <code>number</code> - Elapsed time in milliseconds.  
<a name="module_Timely..Timely"></a>

### Timely~Timely
The Timely class provides methods for managing timers including
setTimeout, setInterval, and setImmediate.

**Kind**: inner class of [<code>Timely</code>](#module_Timely)  

* [~Timely](#module_Timely..Timely)
    * [new Timely([opts], [ctx])](#new_module_Timely..Timely_new)
    * [.getFn(name, clear)](#module_Timely..Timely+getFn) ⇒ <code>function</code>
    * [.setTimeout(name, fn, time)](#module_Timely..Timely+setTimeout) ⇒ <code>this</code>
    * [.setInterval(name, fn, time)](#module_Timely..Timely+setInterval) ⇒ <code>this</code>
    * [.setImmediate(name, fn)](#module_Timely..Timely+setImmediate) ⇒ <code>this</code>
    * [.active(name)](#module_Timely..Timely+active) ⇒ <code>boolean</code>
    * [.clear(...args)](#module_Timely..Timely+clear) ⇒ <code>this</code>
    * [.adjust(name, time)](#module_Timely..Timely+adjust) ⇒ <code>this</code>
    * [.end()](#module_Timely..Timely+end) ⇒ <code>boolean</code>

<a name="new_module_Timely..Timely_new"></a>

#### new Timely([opts], [ctx])
Constructs a new Timely object.


| Param | Type | Default | Description |
| --- | --- | --- | --- |
| [opts] | <code>Object</code> | <code>{}</code> | An options object that includes custom  timer functions. |
| [ctx] | <code>\*</code> |  | The context for 'this'. |

<a name="module_Timely..Timely+getFn"></a>

#### timely.getFn(name, clear) ⇒ <code>function</code>
Retrieves a named function, executes it, and either clears or resets the timer.

**Kind**: instance method of [<code>Timely</code>](#module_Timely..Timely)  
**Returns**: <code>function</code> - Function that will execute the desired action when called.  

| Param | Type | Description |
| --- | --- | --- |
| name | <code>string</code> | Name of the timer function. |
| clear | <code>boolean</code> | Whether to clear or reset the timer. |

<a name="module_Timely..Timely+setTimeout"></a>

#### timely.setTimeout(name, fn, time) ⇒ <code>this</code>
Set a named timeout function.

**Kind**: instance method of [<code>Timely</code>](#module_Timely..Timely)  
**Returns**: <code>this</code> - Current instance.  

| Param | Type | Description |
| --- | --- | --- |
| name | <code>string</code> | Name of the timer function. |
| fn | <code>function</code> | Function to execute after the timeout. |
| time | <code>number</code> | Time in milliseconds after which to execute the function. |

<a name="module_Timely..Timely+setInterval"></a>

#### timely.setInterval(name, fn, time) ⇒ <code>this</code>
Set a named interval function.

**Kind**: instance method of [<code>Timely</code>](#module_Timely..Timely)  
**Returns**: <code>this</code> - Current instance.  

| Param | Type | Description |
| --- | --- | --- |
| name | <code>string</code> | Name of the timer function. |
| fn | <code>function</code> | Function to execute on each interval. |
| time | <code>number</code> | Time in milliseconds for the interval. |

<a name="module_Timely..Timely+setImmediate"></a>

#### timely.setImmediate(name, fn) ⇒ <code>this</code>
Set a named immediate function.

**Kind**: instance method of [<code>Timely</code>](#module_Timely..Timely)  
**Returns**: <code>this</code> - Current instance.  

| Param | Type | Description |
| --- | --- | --- |
| name | <code>string</code> | Name of the immediate function. |
| fn | <code>function</code> | Function to execute immediately. |

<a name="module_Timely..Timely+active"></a>

#### timely.active(name) ⇒ <code>boolean</code>
Check if a timer is active.

**Kind**: instance method of [<code>Timely</code>](#module_Timely..Timely)  
**Returns**: <code>boolean</code> - Whether the timer is active.  

| Param | Type | Description |
| --- | --- | --- |
| name | <code>string</code> | Name of the timer. |

<a name="module_Timely..Timely+clear"></a>

#### timely.clear(...args) ⇒ <code>this</code>
Clear one or more timers.

**Kind**: instance method of [<code>Timely</code>](#module_Timely..Timely)  
**Returns**: <code>this</code> - Current instance.  

| Param | Type | Description |
| --- | --- | --- |
| ...args | <code>string</code> | Names of the timers to clear. |

<a name="module_Timely..Timely+adjust"></a>

#### timely.adjust(name, time) ⇒ <code>this</code>
Adjust the duration of a named timer.

**Kind**: instance method of [<code>Timely</code>](#module_Timely..Timely)  
**Returns**: <code>this</code> - Current instance.  

| Param | Type | Description |
| --- | --- | --- |
| name | <code>string</code> | Name of the timer. |
| time | <code>number</code> | New duration for the timer in milliseconds. |

<a name="module_Timely..Timely+end"></a>

#### timely.end() ⇒ <code>boolean</code>
End all timers and clear the context.

**Kind**: instance method of [<code>Timely</code>](#module_Timely..Timely)  
**Returns**: <code>boolean</code> - Whether the context was cleared.  
<a name="module_Timestamp"></a>

## Timestamp
Timestamp - A module that generates universally unique hex string 
timestamps. The timestamps generated are unique across instances and even on 
the same millisecond. This module is part of the `basd` library, which is a 
foundational lib for many other projects.


* [Timestamp](#module_Timestamp)
    * [~Timestamp](#module_Timestamp..Timestamp)
        * [new Timestamp(separator, prefix, counterLength)](#new_module_Timestamp..Timestamp_new)
        * _instance_
            * [.now()](#module_Timestamp..Timestamp+now) ⇒ <code>string</code>
            * [.parse(ts)](#module_Timestamp..Timestamp+parse) ⇒ <code>number</code>
        * _static_
            * [.mlts](#module_Timestamp..Timestamp.mlts) ⇒ <code>function</code>
            * [.now(opts)](#module_Timestamp..Timestamp.now) ⇒ <code>string</code>
            * [.create(opts)](#module_Timestamp..Timestamp.create) ⇒ <code>function</code>
            * [.parse(ts, separator, prefix)](#module_Timestamp..Timestamp.parse) ⇒ <code>number</code>

<a name="module_Timestamp..Timestamp"></a>

### Timestamp~Timestamp
A class representing a timestamp generator. The generated timestamps consist of 
a hexadecimal representation of the current time in milliseconds and an instance 
counter.

**Kind**: inner class of [<code>Timestamp</code>](#module_Timestamp)  

* [~Timestamp](#module_Timestamp..Timestamp)
    * [new Timestamp(separator, prefix, counterLength)](#new_module_Timestamp..Timestamp_new)
    * _instance_
        * [.now()](#module_Timestamp..Timestamp+now) ⇒ <code>string</code>
        * [.parse(ts)](#module_Timestamp..Timestamp+parse) ⇒ <code>number</code>
    * _static_
        * [.mlts](#module_Timestamp..Timestamp.mlts) ⇒ <code>function</code>
        * [.now(opts)](#module_Timestamp..Timestamp.now) ⇒ <code>string</code>
        * [.create(opts)](#module_Timestamp..Timestamp.create) ⇒ <code>function</code>
        * [.parse(ts, separator, prefix)](#module_Timestamp..Timestamp.parse) ⇒ <code>number</code>

<a name="new_module_Timestamp..Timestamp_new"></a>

#### new Timestamp(separator, prefix, counterLength)
Create a timestamp generator.


| Param | Type | Default | Description |
| --- | --- | --- | --- |
| separator | <code>string</code> | <code>&quot;_&quot;</code> | The separator used in the timestamp. Default is '_'. |
| prefix | <code>string</code> |  | The prefix added before the timestamp. Default is empty. |
| counterLength | <code>number</code> | <code>2</code> | The length of the counter. Not used currently. |

<a name="module_Timestamp..Timestamp+now"></a>

#### timestamp.now() ⇒ <code>string</code>
Generate a new timestamp.

**Kind**: instance method of [<code>Timestamp</code>](#module_Timestamp..Timestamp)  
**Returns**: <code>string</code> - The generated timestamp.  
<a name="module_Timestamp..Timestamp+parse"></a>

#### timestamp.parse(ts) ⇒ <code>number</code>
Parse a timestamp back into the original time in milliseconds.

**Kind**: instance method of [<code>Timestamp</code>](#module_Timestamp..Timestamp)  
**Returns**: <code>number</code> - The original time in milliseconds.  

| Param | Type | Description |
| --- | --- | --- |
| ts | <code>string</code> | The timestamp to be parsed. |

<a name="module_Timestamp..Timestamp.mlts"></a>

#### Timestamp.mlts ⇒ <code>function</code>
Static method to create a timestamp with ':' as the separator.

**Kind**: static property of [<code>Timestamp</code>](#module_Timestamp..Timestamp)  
**Returns**: <code>function</code> - Bound version of the now function with ':' as the separator.  
<a name="module_Timestamp..Timestamp.now"></a>

#### Timestamp.now(opts) ⇒ <code>string</code>
Static method to generate a new timestamp.

**Kind**: static method of [<code>Timestamp</code>](#module_Timestamp..Timestamp)  
**Returns**: <code>string</code> - The generated timestamp.  

| Param | Type | Description |
| --- | --- | --- |
| opts | <code>object</code> | Options for creating the timestamp. |

<a name="module_Timestamp..Timestamp.create"></a>

#### Timestamp.create(opts) ⇒ <code>function</code>
Static method to create a bound version of the now function.

**Kind**: static method of [<code>Timestamp</code>](#module_Timestamp..Timestamp)  
**Returns**: <code>function</code> - Bound version of the now function.  

| Param | Type | Description |
| --- | --- | --- |
| opts | <code>object</code> | Options for creating the timestamp. |

<a name="module_Timestamp..Timestamp.parse"></a>

#### Timestamp.parse(ts, separator, prefix) ⇒ <code>number</code>
Static method to parse a timestamp back into the original time in milliseconds.

**Kind**: static method of [<code>Timestamp</code>](#module_Timestamp..Timestamp)  
**Returns**: <code>number</code> - The original time in milliseconds.  

| Param | Type | Default | Description |
| --- | --- | --- | --- |
| ts | <code>string</code> |  | The timestamp to be parsed. |
| separator | <code>string</code> | <code>&quot;_&quot;</code> | The separator used in the timestamp. Default is '_'. |
| prefix | <code>string</code> |  | The prefix added before the timestamp. |

