const Heap = require('../lib/heap')

describe('Heap', () => {
  let heap = null

  beforeEach(() => heap = new Heap())
  afterEach(() => heap = null)

  it('Should initialize an empty heap', () => {
    expect(heap.length).to.equal(0)
  })

  it('Should insert elements properly', () => {
    heap.insert('test', 1)
    expect(heap.length).to.equal(1)
    expect(heap.peek()).to.equal('test')
    expect(heap.peekKey()).to.equal(1)
  })

  it('Should heapify an array properly', () => {
    heap.heapify([{ key: 1, value: 'first' }, { key: 0, value: 'zero' }])
    expect(heap.length).to.equal(2)
    expect(heap.peek()).to.equal('zero')
    expect(heap.peekKey()).to.equal(0)
  })

  it('Should remove the root element properly', () => {
    heap.insert('first', 1)
    heap.insert('second', 2)
    const removed = heap.remove()
    expect(removed).to.equal('first')
    expect(heap.length).to.equal(1)
  })

  it('Should return null when trying to remove an element from an empty heap', () => {
    const removed = heap.remove()
    expect(removed).to.be.null
  })

  it('Should return null when trying to peek an element from an empty heap', () => {
    const peeked = heap.peek()
    expect(peeked).to.be.null
  })

  it('Should indicate when the heap is empty', () => {
    expect(heap.isEmpty()).to.be.true
    heap.insert('test', 1)
    expect(heap.isEmpty()).to.be.false
  })

  it('Should clear the heap correctly', () => {
    heap.insert('test', 1)
    heap.clear()
    expect(heap.isEmpty()).to.be.true
  })
})
