const Temporal = require('../lib/temporal')

describe('Temporal', () => {
  let temporal


  describe('constructor', () => {
    beforeEach(() => temporal = new Temporal())
    afterEach(() => temporal = null)

    it('should initialize with correct default values', () => {
      expect(temporal.began).to.be.a('number')
      expect(temporal.time).to.equal(0)
      expect(temporal.maxrun).to.equal(0)
      expect(temporal.speed).to.equal(1)
      expect(temporal.running).to.equal(false)
    })
  })

  describe('now', () => {
    beforeEach(() => temporal = new Temporal())
    afterEach(() => temporal = null)

    it('should return current time', () => {
      expect(temporal.now()).to.equal(temporal.began + temporal.time)
    })
  })

  describe('start', () => {
    beforeEach(() => temporal = new Temporal())
    afterEach(() => temporal = null)

    it('should start the temporal instance', async () => {
      temporal.delay(0, () => {})
      temporal.start()
      expect(temporal.running).to.equal(true)
    })
  })

  describe('run', () => {
    beforeEach(() => temporal = new Temporal())
    afterEach(() => temporal = null)

    it('should run the temporal instance', () => {
      // @todo
    })
  })

  describe('exec', () => {
    beforeEach(() => temporal = new Temporal())
    afterEach(() => temporal = null)

    it('should execute a function immediately', async () => {
      let executed = false
      temporal.exec(() => { executed = true })
      temporal.start()
      expect(executed).to.equal(true)
    })
  })

  describe('delay', () => {
    beforeEach(() => temporal = new Temporal())
    afterEach(() => temporal = null)

    it('should execute a function after a delay', async (done) => {
      let executed = false
      temporal.delay(1000, () => { executed = true })
      temporal.start(0)
      expect(executed).to.equal(true)
    })
  })

  describe('tick', () => {
    it('should execute a function repeatedly', async () => {
      const temporal = new Temporal()
      let counter = 0
      await new Promise(resolve => {
        temporal.tick(100, (...args) => {
          counter++
          if (counter === 3)
            temporal.stop()
            resolve()
        })
        temporal.start(0)
      })
    })
  })

  describe('stop', () => {
    beforeEach(() => temporal = new Temporal())
    afterEach(() => temporal = null)

    it('should stop the temporal instance', () => {
      temporal.tick(100, () => {})
      temporal.start()
      temporal.stop()
      expect(temporal.running).to.equal(false)
    })
  })
})
