const Timely = require('../lib/timely')

describe('Timer Class', () => {
  let timer = null

  beforeEach(() => {
    timer = new Timely.Timer(() => {}, () => {}, 1000, () => {})
  })

  it('should create a new Timer instance', () => {
    expect(timer).to.be.instanceOf(Timely.Timer)
  })

  describe('remaining()', () => {
    it('should return correct remaining time', () => {
      expect(timer.remaining()).to.be.above(990)
    })
  })

  describe('taken()', () => {
    it('should return correct taken time', async () => {
      return new Promise(resolve => {
        setTimeout(() => {
          expect(timer.taken()).to.be.at.least(50)
          resolve()
        }, 50)
      })
    })
  })
})

describe('Timely Class', () => {
  let timely = null

  beforeEach(() => timely = new Timely())
  afterEach(() => {
    timely.clear()
    timely = null
  })

  it('should create a new Timely instance', () => {
    expect(timely).to.be.instanceOf(Timely)
  })

  describe('setTimeout()', () => {
    beforeEach(() => timely = new Timely())
    afterEach(() => {
      timely.clear()
      timely = null
    })
    it('should set a new timeout', async () => {
      timely.setTimeout('testTimeout', () => {}, 10)
      expect(timely.active('testTimeout')).to.be.true
    })
  })

  describe('setInterval()', () => {
    beforeEach(() => timely = new Timely())
    afterEach(() => {
      timely.clear()
      timely = null
    })
    it('should set a new interval', () => {
      timely.setInterval('testInterval', () => {}, 10)
      expect(timely.active('testInterval')).to.be.true
    })
  })

  describe('setImmediate()', () => {
    beforeEach(() => timely = new Timely())
    afterEach(() => {
      timely.clear()
      timely = null
    })
    it('should set a new immediate function', () => {
      timely.setImmediate('testImmediate', () => {})
      expect(timely.active('testImmediate')).to.be.true
    })
  })

  describe('clear()', () => {
    beforeEach(() => timely = new Timely())
    afterEach(() => {
      timely.clear()
      timely = null
    })
    it('should clear a timeout', () => {
      timely.setTimeout('testClear', () => {}, 10)
      timely.clear('testClear')
      expect(timely.active('testClear')).to.be.false
    })
  })

  describe('active()', () => {
    beforeEach(() => timely = new Timely())
    afterEach(() => {
      timely.clear()
      timely = null
    })
    it('should check if a timer is active', () => {
      timely.setTimeout('testActive', () => {}, 10)
      expect(timely.active('testActive')).to.be.true
    })
  })

  describe('adjust()', () => {
    beforeEach(() => timely = new Timely())
    afterEach(() => {
      timely.clear()
      timely = null
    })
    it('should adjust a timeout', async () => {
      return new Promise(resolve => {
        let counter = 0
        timely.setTimeout('testAdjust', () => {
          counter++
        }, 200)
        timely.adjust('testAdjust', 100)
        setTimeout(() => {
          expect(counter).to.equal(1)
          resolve()
        }, 300)
      })
    })
  })

  describe('end()', () => {
    beforeEach(() => timely = new Timely())
    afterEach(() => {
      timely.clear()
      timely = null
    })
    it('should clear all timers and return true', () => {
      timely.setTimeout('testEnd1', () => {}, 10)
      timely.setTimeout('testEnd2', () => {}, 10)
      const result = timely.end()
      expect(timely.active('testEnd1')).to.be.false
      expect(timely.active('testEnd2')).to.be.false
      expect(result).to.be.true
    })
  })
})
