const Timestamp = require('../lib/timestamp')

describe('Timestamp', () => {
  describe('constructor', () => {
    afterEach(() => Timestamp.instance = null) // remove singleton

    it('should create an instance with default parameters', () => {
      const timestamp = new Timestamp()
      expect(timestamp).to.be.an.instanceof(Timestamp)
      expect(timestamp.separator).to.equal('_')
      expect(timestamp.prefix).to.equal('')
    })

    it('should create an instance with given parameters', () => {
      const timestamp = new Timestamp(':', 'prefix')
      expect(timestamp).to.be.an.instanceof(Timestamp)
      expect(timestamp.separator).to.equal(':')
      expect(timestamp.prefix).to.equal('prefix')
    })
  })

  describe('now', () => {
    it('should return a timestamp', () => {
      const timestamp = new Timestamp()
      const now = timestamp.now()
      expect(now).to.be.a('string')
    })

    it('should return unique timestamps for consecutive calls', () => {
      const timestamp = new Timestamp()
      const now1 = timestamp.now()
      const now2 = timestamp.now()
      expect(now1).to.not.equal(now2)
    })
  })

  describe('static now', () => {
    it('should return a timestamp', () => {
      const now = Timestamp.now()
      expect(now).to.be.a('string')
    })
  })

  describe('static create', () => {
    it('should return a function that generates timestamps', () => {
      const now = Timestamp.create()
      expect(now).to.be.a('function')
      const ts = now()
      expect(ts).to.be.a('string')
    })
  })

  describe('parse', () => {
    it('should parse a timestamp back into the original time', () => {
      const timestamp = new Timestamp()
      const now = timestamp.now()
      const parsed = timestamp.parse(now)
      expect(parsed).to.be.a('number')
    })
  })

  describe('static parse', () => {
    it('should parse a timestamp back into the original time', () => {
      const timestamp = new Timestamp()
      const now = timestamp.now()
      const parsed = Timestamp.parse(now)
      expect(parsed).to.be.a('number')
    })
  })
})
